@@ -1,5 +1,5 @@
 #
-#   xts: eXtensible time-series 
+#   xts: eXtensible time-series
 #
 #   Copyright (C) 2008  Jeffrey A. Ryan jeff.a.ryan @ gmail.com
 #
@@ -20,7 +20,7 @@
 
 
 na.omit.xts <- function(object, ...) {
-  xx <- .Call('na_omit_xts', object, PACKAGE="xts")
+  xx <- .Call(C_na_omit_xts, object)
   if(length(xx)==0)
     return(structure(xts(,),.Dim=c(0,NCOL(object))))
   naa <- attr(xx,'na.action')
@@ -41,7 +41,7 @@ na.omit.xts <- function(object, ...) {
 }
 
 na.exclude.xts <- function(object, ...) {
-  xx <- .Call('na_omit_xts', object, PACKAGE="xts")
+  xx <- .Call(C_na_omit_xts, object)
   naa <- attr(xx,'na.action')
   if(length(naa) == 0)
     return(xx)
@@ -92,32 +92,64 @@ na.replace <- function(x) {
   rbind(x,tmp)
 }
 
+
+#' Last Observation Carried Forward
+#' 
+#' \pkg{xts} method replace `NA` with most recent non-NA
+#' 
+#' This is the \pkg{xts} method for the S3 generic `na.locf()`. The primary
+#' difference to note is that after the `NA` fill action is carried out, the
+#' default it to leave trailing or leading `NA`'s in place. This is different
+#' than \pkg{zoo} behavior.
+#' 
+#' @param object An xts object.
+#' @param na.rm Logical indicating whether leading/trailing `NA` should be
+#'   removed. The default is `FALSE` unlike the zoo method.
+#' @param fromLast Logical indicating whether observations should be carried
+#'   backward rather than forward. Default is `FALSE`.
+#' @param maxgap Consecutive runs of observations more than 'maxgap' will
+#'   remain `NA`. See [`na.locf()`][zoo::zoo] for details.
+#' @param \dots Unused.
+#' 
+#' @return An object where each `NA` in `object` is replaced by the most recent
+#'   non-NA prior to it. See [`na.locf()`][zoo::zoo] for details.
+#' 
+#' @author Jeffrey A. Ryan
+#' 
+#' @seealso [`na.locf()`][zoo::zoo]
+#' 
+#' @keywords misc
+#' @examples
+#' 
+#' x <- xts(1:10, Sys.Date()+1:10)
+#' x[c(1,2,5,9,10)] <- NA
+#' 
+#' x
+#' na.locf(x)
+#' na.locf(x, fromLast=TRUE)
+#' na.locf(x, na.rm=TRUE, fromLast=TRUE)
+#' 
 na.locf.xts <- function(object, na.rm=FALSE, fromLast=FALSE, maxgap=Inf, ...) {
-    stopifnot(is.xts(object))
     maxgap <- min(maxgap, NROW(object))
     if(length(object) == 0)
       return(object)
-    if(dim(object)[2] > 1) {
-      x <- object
-      for(n in 1:NCOL(object))
-        x[,n] <- .Call('na_locf', object[,n], fromLast, maxgap, Inf, PACKAGE='xts')
-      #.xts(apply(object, 2, function(x) .Call('na_locf', x, fromLast, maxgap, PACKAGE='xts')),
-      #     .index(object), tzone=indexTZ(object), .indexCLASS=indexClass(object))
-    } else {
-      x <- .Call("na_locf", object, fromLast, maxgap, Inf, PACKAGE="xts")
-    }
+    if(hasArg("x") || hasArg("xout"))
+      return(NextMethod(.Generic))
+    x <- .Call(C_na_locf, object, fromLast, maxgap, Inf)
     if(na.rm) {
       return(structure(na.omit(x),na.action=NULL))
     } else x
 }
 
-.na.locf.xts <- function(object, na.rm=FALSE, fromLast=FALSE, maxgap=Inf, ...) {
-    stopifnot(is.xts(object))
-    maxgap <- min(maxgap, NROW(object))
-    if(length(object) == 0)
-      return(object)
-    x <- .Call("na_locf_col", object, fromLast, maxgap, Inf, PACKAGE="xts")
-    if(na.rm) {
-      return(structure(na.omit(x),na.action=NULL))
-    } else x
+na.fill.xts <- function(object, fill, ix, ...) {
+  if (length(fill) == 1 && missing(ix)) {
+    # na.fill0() may change the storage type of 'object'
+    # make sure 'fill' argument is same type as 'object'
+    fill. <- fill
+    storage.mode(fill.) <- storage.mode(object)
+
+    return(na.fill0(object, fill.))
+  } else {
+    NextMethod(.Generic)
+  }
 }
